#include "variantmaptablemodel.h"
#include <QJsonArray>
#include <QJsonObject>
#include <QJsonValue>
#include <QDebug>


VariantMapTableModel::VariantMapTableModel(QObject *parent) : QAbstractTableModel(parent)
{

}

void VariantMapTableModel::registerColumn(AbstractColumn *column)
{
    _columns.append(column);
}

void VariantMapTableModel::addRow(QVariantMap rowData)
{
    int id = rowData.value(PRM_ID).toInt();
    beginInsertRows(QModelIndex(), _rowIndex.count(), _rowIndex.count());
    _rowIndex.append(id);
    _dataHash.insert(id, rowData);
    endInsertRows();
}

int VariantMapTableModel::idByRow(int row) const
{
    return _rowIndex.at(row);
}

int VariantMapTableModel::rowById(int id) const
{
    return _rowIndex.indexOf(id);
}

int VariantMapTableModel::colByName(QString name) const
{
    for(int col = 0; col < _columns.count(); col++) {
        if (nameByCol(col) == name)
            return col;
    }
    return -1;
}

QString VariantMapTableModel::nameByCol(int col) const
{
    return _columns.at(col)->name();
}

int VariantMapTableModel::rowCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent)
    return _rowIndex.count();
}

int VariantMapTableModel::columnCount(const QModelIndex &parent) const
{
    Q_UNUSED(parent)
    return _columns.count();
}

QVariant VariantMapTableModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();
    int id = idByRow(index.row());
    if (role == Qt::UserRole) {
        return id;
    }
    QVariantMap rowData = _dataHash.value(id);
    return _columns.at(index.column())->colData(rowData, role);
}

bool VariantMapTableModel::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if (!index.isValid())
        return false;
    if (role == Qt::EditRole) {
       int id = idByRow(index.row());
       QString col = nameByCol(index.column());
       _dataHash[id].insert(col, value);
       emit dataChanged(index, index);
       for(int colIdx=0; colIdx < columnCount(); colIdx++) {
           if (!_columns.at(colIdx)->editable())
               emit dataChanged(this->index(index.row(), colIdx), this->index(index.row(), colIdx));
       }
    }
    return false;
}


SimpleColumn::SimpleColumn(QString name) : AbstractColumn(name)
{
}

QVariant SimpleColumn::colData(const QVariantMap &rowData, int role)
{
    if (role != Qt::DisplayRole && role != Qt::EditRole)
        return QVariant();
    return rowData.value(name());
}

AbstractColumn::AbstractColumn(QString name) : _name(name)
{
}

AbstractColumn::~AbstractColumn()
{
}

LookupColumn::LookupColumn(QString name, VariantMapTableModel *model, QString keyCol, QString lookupCol)
    : AbstractColumn(name), _lookupModel(model), _keyColumn(keyCol), _lookupColumn(lookupCol)
{
    _editable = false;
}

QVariant LookupColumn::colData(const QVariantMap &rowData, int role)
{
    if (role == Qt::DisplayRole) {
        int key = rowData.value(_keyColumn).toInt();
        int lookupRow = _lookupModel->rowById(key);
        if (lookupRow < 0)
            return QVariant();
        int lookupCol = _lookupModel->colByName(_lookupColumn);
        return _lookupModel->index(lookupRow, lookupCol).data();
    }
    return QVariant();
}
